import { convertRGBColorByPixel, convertRGBColorByPlane, convertYBRFullByPixel, convertYBRFullByPlane, convertPALETTECOLOR } from './imageLoader/colorSpaceConverters/index';
import { default as wadouri } from './imageLoader/wadouri/index';
import { default as wadors } from './imageLoader/wadors/index';
import { default as init } from './init';
import { default as convertColorSpace } from './imageLoader/convertColorSpace';
import { default as createImage } from './imageLoader/createImage';
import { default as decodeJPEGBaseline8BitColor } from './imageLoader/decodeJPEGBaseline8BitColor';
import { default as getImageFrame } from './imageLoader/getImageFrame';
import { default as getMinMax } from './shared/getMinMax';
import { default as isColorImage } from './shared/isColorImage';
import { default as isJPEGBaseline8BitColor } from './imageLoader/isJPEGBaseline8BitColor';
import { default as getPixelData } from './imageLoader/wadors/getPixelData';
import { internal } from './imageLoader/internal/index';
import * as constants from './constants';
import type * as Types from './types';
import { decodeImageFrame } from './decodeImageFrameWorker';
declare const cornerstoneDICOMImageLoader: {
    constants: typeof constants;
    convertRGBColorByPixel: typeof convertRGBColorByPixel;
    convertRGBColorByPlane: typeof convertRGBColorByPlane;
    convertYBRFullByPixel: typeof convertYBRFullByPixel;
    convertYBRFullByPlane: typeof convertYBRFullByPlane;
    convertPALETTECOLOR: typeof convertPALETTECOLOR;
    wadouri: {
        metaData: {
            getImagePixelModule: typeof import("./imageLoader/wadouri/metaData").getImagePixelModule;
            getLUTs: typeof import("./imageLoader/wadouri/metaData").getLUTs;
            getModalityLUTOutputPixelRepresentation: typeof import("./imageLoader/wadouri/metaData").getModalityLUTOutputPixelRepresentation;
            getNumberValues: typeof import("./imageLoader/wadouri/metaData").getNumberValues;
            metaDataProvider: typeof import("./imageLoader/wadouri/metaData").metaDataProvider;
            metadataForDataset: typeof import("./imageLoader/wadouri/metaData").metadataForDataset;
        };
        dataSetCacheManager: {
            isLoaded: (uri: string) => boolean;
            load: (uri: string, loadRequest: Types.LoadRequestFunction, imageId: string) => import("./imageLoader/wadouri/dataSetCacheManager").CornerstoneWadoLoaderCachedPromise;
            unload: (uri: string) => void;
            getInfo: typeof import("./imageLoader/wadouri/dataSetCacheManager").getInfo;
            purge: () => void;
            get: (uri: string) => import("dicom-parser").DataSet;
            update: (uri: string, dataSet: import("dicom-parser").DataSet) => void;
        };
        fileManager: {
            add: (file: Blob) => string;
            get: (index: number) => Blob;
            remove: (index: number) => void;
            purge: () => void;
        };
        getEncapsulatedImageFrame: typeof import("./imageLoader/wadouri/getEncapsulatedImageFrame").default;
        getUncompressedImageFrame: typeof import("./imageLoader/wadouri/getUncompressedImageFrame").default;
        loadFileRequest: typeof import("./imageLoader/wadouri/loadFileRequest").default;
        loadImageFromPromise: typeof import("./imageLoader/wadouri/loadImage").loadImageFromPromise;
        getLoaderForScheme: typeof import("./imageLoader/wadouri/loadImage").getLoaderForScheme;
        getPixelData: typeof import("./imageLoader/wadouri/getPixelData").default;
        loadImage: typeof import("./imageLoader/wadouri/loadImage").loadImage;
        parseImageId: typeof import("./imageLoader/wadouri/parseImageId").default;
        unpackBinaryFrame: typeof import("./imageLoader/wadouri/unpackBinaryFrame").default;
        register: typeof import("./imageLoader/wadouri/register").default;
    };
    wadors: {
        metaData: {
            getNumberString: typeof import("./imageLoader/wadors/metaData").getNumberString;
            getNumberValue: typeof import("./imageLoader/wadors/metaData").getNumberValue;
            getNumberValues: typeof import("./imageLoader/wadors/metaData").getNumberValues;
            getValue: typeof import("./imageLoader/wadors/metaData").getValue;
            metaDataProvider: typeof import("./imageLoader/wadors/metaData").metaDataProvider;
        };
        findIndexOfString: typeof import("./imageLoader/wadors/findIndexOfString").default;
        getPixelData: typeof getPixelData;
        loadImage: typeof import("./imageLoader/wadors/loadImage").default;
        metaDataManager: {
            add: (imageId: string, metadata: Types.WADORSMetaData) => void;
            get: (imageId: string) => Types.WADORSMetaData;
            remove: (imageId: any) => void;
            purge: () => void;
        };
        register: typeof import("./imageLoader/wadors/register").default;
    };
    init: typeof init;
    convertColorSpace: typeof convertColorSpace;
    createImage: typeof createImage;
    decodeJPEGBaseline8BitColor: typeof decodeJPEGBaseline8BitColor;
    getImageFrame: typeof getImageFrame;
    getPixelData: typeof getPixelData;
    getMinMax: typeof getMinMax;
    isColorImage: typeof isColorImage;
    isJPEGBaseline8BitColor: typeof isJPEGBaseline8BitColor;
    internal: {
        xhrRequest: typeof import("./imageLoader/internal/xhrRequest").default;
        streamRequest: typeof import("./imageLoader/internal/streamRequest").default;
        setOptions: typeof import("./imageLoader/internal/options").setOptions;
        getOptions: typeof import("./imageLoader/internal/options").getOptions;
    };
    decodeImageFrame: typeof decodeImageFrame;
};
export { constants, convertRGBColorByPixel, convertRGBColorByPlane, convertYBRFullByPixel, convertYBRFullByPlane, convertPALETTECOLOR, wadouri, wadors, init, convertColorSpace, createImage, decodeJPEGBaseline8BitColor, getImageFrame, getPixelData, getMinMax, isColorImage, isJPEGBaseline8BitColor, internal, decodeImageFrame, };
export type { Types };
export default cornerstoneDICOMImageLoader;
